import { ACTIVITY_NAME, INGAME_SCORE_KEY, RANK_SCORE_SCALE } from 'common/Constants'
import { SyncLocker } from 'common/SyncLocker'
import { formatNumShow } from 'common/Utils'
import logger from 'logger/logger'
import { join } from 'path'
import { ActivityUser } from 'models/ActivityUser'
import { DrawRecord } from 'models/DrawRecord'
import { InGameScoreRecord } from 'models/InGameScoreRecord'
import { InGameTaskRecord } from 'models/InGameTaskRecord'
import { rankLevel } from 'services/rank.svr'
import { formatDate } from 'utils/utcdate.util'
import { BaseController, ROLE_ANON, ZError, ZRedisClient, role, router } from 'zutils'
import { formatAddress } from 'zutils/utils/chain.util'
import { InGameStats } from 'models/InGameStats'
import { drawCfgs, drawOnce, queryInGameInfo } from 'services/game.svr'
import { updateRankScore } from 'services/ingame.rank.svr'
import { NftHolder } from 'models/chain/NftHolder'
import { tokenRarity } from 'taskingame/NftHolderCheck'
const fs = require('fs')

const sourceList = require('../../configs/ingame_tasks.json')
const taskMap = new Map()
for (let o of sourceList) {
  taskMap.set(o.id, o)
}
const taskListStr = JSON.stringify(sourceList)

const initTasks = () => {
  const tasks = join(__dirname, '../taskingame')
  const list = new Map()
  fs.readdirSync(tasks)
    .filter((file: string) => ~file.search(/^[^.].*\.(ts|js)$/))
    .forEach((file: any) => {
      const Task = require('../taskingame/' + file)
      list.set(Task.default.name, Task)
    })
  return list
}
const allTasks = initTasks()
/**
 * 游戏内任务相关接口
 */
const MAX_LIMIT = 100
class InGameController extends BaseController {
  /**
   * 任务列表
   */
  @role(ROLE_ANON)
  @router('get /api/ingame/tasks')
  async taskList(req) {
    const user = req.user
    const now = Date.now()
    let activity = req.activity
    let list = JSON.parse(taskListStr)
    let gameData = {
      loginVal: '0',
      battleTimes: 0,
      winTimes: 0,
      kills: 0,
      getGoldVal: '0',
    }
    for (let o of list) {
      if (o.checkTime) {
        o.status = now < activity.drawTime ? -1 : 0
      } else {
        o.status = 0
      }
    }
    if (user) {
      if (user.gameAccountBinded()) {
        try {
          if (user.googleId) {
            let res = await queryInGameInfo(user.googleId, '0')
            Object.assign(gameData, res)
          } else if (user.emailId) {
            let res = await queryInGameInfo(user.emailId, '6')
            Object.assign(gameData, res)
          } else if (user.clientId) {
            let res = await queryInGameInfo(user.clientId, user.clientPlat)
            Object.assign(gameData, res)
          }
        } catch (e) {
          logger.info('queryInGameInfo with err: ', e.message || e)
        }
      }
      const dateTag = formatDate(new Date())
      let records = await InGameTaskRecord.find({ user: user.id, dateTag })
      let recordSet = new Set()
      for (let record of records) {
        recordSet.add(record.task)
      }
      for (let sub of list) {
        const Task = allTasks.get(sub.task)
        const taskInstance = new Task.default({ user, activity: req.activity })
        const checkResult = await taskInstance.check(sub, gameData)
        if (sub.status >= 0 && !!checkResult) {
          sub.status = 2
          if (sub.task === 'GoogleConnect') {
            sub.status = 3
          } else if (sub.task === 'NftHolderCheck') {
            sub.ticket = checkResult
          }
        }
        sub.status = recordSet.has(sub.id) ? 3 : sub.status === 0 ? 1 : sub.status
      }
    }
    return list
  }
  /**
   * 领取任务奖励
   */
  @router('post /api/ingame/claim')
  async claimInGameReward(req) {
    await new SyncLocker().checkLock(req)
    logger.db('claim_ingame', req)
    const { task } = req.params
    const user = req.user
    if (!user.gameAccountBinded()) {
      throw new ZError(10, 'need connect game account first')
    }
    if (!taskMap.has(task)) {
      throw new ZError(11, 'task not found')
    }
    let cfg = taskMap.get(task)
    if (!allTasks.has(cfg.task)) {
      throw new ZError(12, 'task not found')
    }
    const dateTag = formatDate(new Date())
    let record = await InGameTaskRecord.findOne({ user: user.id, task, dateTag })
    if (record) {
      throw new ZError(13, 'already claimed today')
    }
    const Task = allTasks.get(cfg.task)
    const taskInstance = new Task.default({ user, activity: req.activity })
    const { score, ticket } = await taskInstance.claimReward(cfg)
    let recordNew = new InGameTaskRecord({
      user: user.id,
      task,
      dateTag,
      score,
      ticket,
    })
    await recordNew.save()
    return { score, ticket, status: 3 }
  }
  /**
   * 游戏内抽奖
   */
  @router('post /api/ingame/draw')
  async draw(req) {
    await new SyncLocker().checkLock(req)
    logger.db('draw_ingame', req)
    const user = req.user
    let { step } = req.params
    if (isNaN(step)) {
      throw new ZError(10, 'params mismatch')
    }
    step = parseInt(step || '1')
    if (step < 1) {
      step = 1
    }
    if (!user.gameAccountBinded()) {
      throw new ZError(10, 'need connect game account first')
    }
    const ingameStat = await InGameStats.insertOrUpdate({ user: user.id }, {})
    if (ingameStat.ticket < step) {
      throw new ZError(11, 'no ticket')
    }
    // TODO:: 抽奖逻辑
    let saveRes = await InGameStats.findOneAndUpdate(
      { user: user.id, ticket: { $gte: step } },
      { $inc: { ticket: -step } },
      {
        new: false,
        upsert: false,
        includeResultMetadata: true,
      },
    )
    // @ts-ignore
    if (!saveRes.ok || saveRes.lastErrorObject.n < 1) {
      throw new ZError(11, 'no ticket.')
    }
    let score = 0
    let items: any[] = []
    let position: number[] = []
    const dateTag = formatDate(new Date())
    for (let i = 0; i < step; i++) {
      let reward = await drawOnce(user.id, user.inWhiteList)
      let record = new DrawRecord({
        user: user.id,
        score: 0,
      })
      if (reward.type === 0) {
        score += reward.amount
        record.score = reward.amount
        await updateRankScore({
          user: user.id,
          score: reward.amount,
          scoreType: 'draw_ingame',
          scoreParams: {
            date: dateTag,
            position: reward.position,
          },
        })
      } else {
        items.push(reward)
        record.items = [reward]
        if (reward.type === 1) {
          await ActivityUser.updateOne({ _id: user.id }, { inWhiteList: true, $inc: { whiteListNum: 1 } })
        }
      }
      await record.save()
      position.push(reward.position)
    }
    return { score, items, position }
  }

  /**
   * 游戏内抽奖记录
   */
  @router('get /api/ingame/draw_history')
  async history(req) {
    const user = req.user
    let records = await DrawRecord.find({ user: user.id }).sort({ _id: -1 })
    return records.map(record => {
      return {
        score: formatNumShow(record.score),
        // @ts-ignore
        time: record.createdAt.getTime(),
        items: record.items,
      }
    })
  }

  /**
   * 游戏内积分列表
   */
  @router('get /api/ingame/score_list')
  async scoreList(req) {
    const user = req.user
    let records = await InGameScoreRecord.find({ user: user.id }).sort({ _id: -1 })
    return records.map(record => {
      return {
        score: formatNumShow(record.score),
        // @ts-ignore
        time: record.createdAt.getTime(),
        type: record.type,
      }
    })
  }
  /**
   * 游戏内积分排行榜
   */
  @role(ROLE_ANON)
  @router('get /api/ingame/leaderboard/:page')
  async leaderboard(req) {
    let { page, limit } = req.params
    page = parseInt(page || '0')
    limit = parseInt(limit || MAX_LIMIT)
    page = page < 0 ? 0 : page
    const start = page * limit
    const end = start + limit - 1
    const records = await new ZRedisClient().zrevrange(INGAME_SCORE_KEY, start, end)
    let results: any = []
    for (let i = 0; i < records.length; i += 2) {
      const id = records[i]
      let score = formatNumShow(parseInt(records[i + 1]) / RANK_SCORE_SCALE)
      const user = await ActivityUser.findById(id)
      const rank = start + i / 2 + 1
      results.push({
        rank,
        level: rankLevel(rank),
        nickname: user?.twitterName || user?.discordName || user?.address ? formatAddress(user.address) : 'unknown',
        avatar: user?.twitterAvatar || '',
        score,
      })
    }
    return results
  }

  @role(ROLE_ANON)
  @router('get /api/ingame/draw_cfg')
  async drawCfg(req) {
    let cfgs = drawCfgs
    let results = []
    for (let cfg of cfgs) {
      results.push({
        position: cfg.position,
        type: cfg.type,
        amount: cfg.amount,
        name: cfg.name,
      })
    }
    return results
  }

  @router('get /api/ingame/nft_list')
  async nftList(req) {
    let user = req.user
    let records = await NftHolder.find({
      chain: '1',
      address: '0xec23679653337d4c6390d0eeba682246a6067777',
      user: user.address.toLowerCase(),
    })
    return records.map(record => {
      return {
        tokenId: record.tokenId,
        rarity: tokenRarity(record.tokenId),
      }
    })
  }
}
