import { STEP_CHEST_RATE, STEP_SCORE_MAX, STEP_SCORE_MIN } from 'common/Constants'
import { ActivityChest, ActivityChestClass, ChestStatusEnum } from 'models/ActivityChest'
import { ZError, ZRedisClient } from 'zutils'
import { DocumentType } from '@typegoose/typegoose'
import { timeoutFetch } from 'zutils/utils/net.util'
import { RewardRecord } from 'models/RewardRecrd'
import logger from 'logger/logger'

export const drawCfgs = require('../../configs/draw_cfg.json')
// 处理draw_cfg.json中的probability, 计算出每个奖励的概率区间
let probability = 0
for (let i = 0; i < drawCfgs.length; i++) {
  let cfg = drawCfgs[i]
  probability += cfg.probability
  cfg.probability = probability
}

const chestCfg = require('../../configs/chest.json')
const chestBonusItems = require('../../configs/chest_bonus_item.json')
const chestLevelMap = new Map()

let STEP_CHEST_LEVEL = chestCfg.chests.map((cfg: any) => cfg.probability)
STEP_CHEST_LEVEL = STEP_CHEST_LEVEL.sort((a: number, b: number) => a - b)
for (let i = 1; i < STEP_CHEST_LEVEL.length; i++) {
  STEP_CHEST_LEVEL[i] += STEP_CHEST_LEVEL[i - 1]
}

chestCfg.chests.forEach((cfg: any) => {
  chestLevelMap.set(cfg.level, cfg)
})

const generateBounsCfg = (cfg: any) => {
  let randoms = []
  for (let i = 0; i < cfg.maxBounsCount; i++) {
    randoms.push(Math.random())
  }
  let total = randoms.reduce((a: number, b: number) => a + b)
  randoms = randoms.map((r: number) => r / total)
  let ys = []
  randoms.forEach((r: number) => {
    let tmp = cfg.maxBounsScore * r
    tmp = Math.max(1, Math.round(tmp))
    ys.push(tmp)
  })
  ys.sort((a: number, b: number) => b - a)
  return ys
}

export const generateNewChest = (uid: string, activity: string, level = 1, status = ChestStatusEnum.LOCKED) => {
  let cfg = chestLevelMap.get(level)
  if (!cfg) {
    throw new ZError(11, 'chest cfg not found')
  }
  let scoreInit = Math.floor(Math.random() * (cfg.initScoreMax - cfg.initScoreMin + 1) + cfg.initScoreMin)
  let chest = new ActivityChest({
    user: uid,
    activity: activity,
    level: level,
    maxBounsCount: cfg.maxBounsCount,
    bounsCfg: generateBounsCfg(cfg),
    scoreInit,
    status,
  })
  return chest
}

export const generateChestLevel = function (): number {
  const levelDefine = STEP_CHEST_LEVEL
  let randomLevel = Math.floor(Math.random() * levelDefine[levelDefine.length - 1])
  let level = 0
  for (let i = 0; i < levelDefine.length; i++) {
    if (randomLevel < levelDefine[i]) {
      level = i
      break
    }
  }
  return levelDefine.length - level
}
/**
 * 生成步数奖励
 * // TODO:: 增加白名单和特殊种类nft的奖励
 * @param uid
 * @param activity
 * @param step
 * @returns
 */
export const generateStepReward = (uid: string, activity: string, step: number) => {
  let score = 0
  let chests = []
  for (let i = 0; i < step; i++) {
    if (Math.random() < STEP_CHEST_RATE) {
      let randomLevel = generateChestLevel()
      let chest = generateNewChest(uid, activity, randomLevel, ChestStatusEnum.NORMAL)
      chests.push(chest)
    } else {
      score += Math.floor(Math.random() * (STEP_SCORE_MAX - STEP_SCORE_MIN + 1) + STEP_SCORE_MIN)
    }
  }
  return { score, chests }
}

export const generateChestBonus = async (chest: DocumentType<ActivityChestClass>, hasWhite: boolean) => {
  let rewards = []
  // 如果宝箱的助力用户数量小于最大助力次数, 则不生成奖励
  if (process.env.NODE_ENV === 'production' && chest.bonusUsers.length < chest.maxBounsCount) {
    return []
  }
  let bonusItem
  for (let i = 0; i < chestBonusItems.length; i++) {
    let item = chestBonusItems[i]
    if (Math.random() < item.probability / 100) {
      if (item.type === 1) {
        continue
      }
      bonusItem = item
      break
    }
  }
  if (!bonusItem) {
    return []
  }
  const redisKey = `chest_bonus_${bonusItem.type}`
  const itemId = await new ZRedisClient().spop(redisKey)
  if (!itemId) {
    return []
  }
  rewards.push({
    id: itemId,
    type: bonusItem.type,
    name: bonusItem.name,
    desc: bonusItem.desc,
    amount: 1,
  })
  return rewards
}

export const queryInGameInfo = async (openId: string, channel: string) => {
  let url = `${process.env.GAME_SERVER_URL}&channel=${channel}&openId=${openId}`
  const key = `gameinfo:${openId}:${channel}`
  const cache = await new ZRedisClient().get(key)
  if (cache) {
    try {
      let result = JSON.parse(cache)
      return result
    } catch (e) {
      console.error(e)
    }
  }
  let res = await timeoutFetch(url, {}, 5000)
  let data = await res.json()
  if (data.errcode) {
    throw new ZError(data.errcode, data.errmsg)
  }
  let result = data.info
  new ZRedisClient().pub.set(key, JSON.stringify(result), 'EX', 60, () => {
    console.log('cache set success: ', key)
  })
  return result
}

/**
 * 根据drawCfgs中设定的probability, 抽取指定的奖励
 * 如果抽到的type为1或2, 则从redis中spop一个值, 如果该值为空, 则再次调用本方法, 直到抽到奖励位置
 * */
export const drawOnce = async (userId: string, hasWhite: boolean) => {
  let random = Math.random() * probability
  let reward
  for (let i = 0; i < drawCfgs.length; i++) {
    let cfg = drawCfgs[i]
    if (random < cfg.probability) {
      // deep copy
      reward = JSON.parse(JSON.stringify(cfg))
      break
    }
  }
  if (!reward) {
    return drawOnce(userId, hasWhite)
  }
  if (reward.type === 1 || reward.type === 2) {
    const redisKey = `draw_${reward.type}`
    const itemId = await new ZRedisClient().spop(redisKey)
    if (!itemId) {
      return drawOnce(userId, hasWhite)
    }
    // double check if the record is exist
    let record = await RewardRecord.findById(itemId)
    if (!record) {
      logger.info('record not found: ', itemId)
      return drawOnce(userId, hasWhite)
    }
    record.user = userId
    await record.save()
    reward.id = itemId
  }
  return reward
}
